"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.PrivateFilesAssetProcessor = void 0;
var _handlerGraphql = require("@webiny/handler-graphql");
var _NotAuthorizedOutputStrategy = require("./NotAuthorizedOutputStrategy");
var _RedirectToPublicUrlOutputStrategy = require("./RedirectToPublicUrlOutputStrategy");
var _RedirectToPrivateUrlOutputStrategy = require("./RedirectToPrivateUrlOutputStrategy");
var _PrivateCache = require("./PrivateCache");
var _PublicCache = require("./PublicCache");
var _apiHeadlessCms = require("@webiny/api-headless-cms");
class PrivateFilesAssetProcessor {
  constructor(context, assetAuthorizer, assetProcessor) {
    this.assetAuthorizer = assetAuthorizer;
    this.context = context;
    this.assetProcessor = assetProcessor;
  }
  async process(assetRequest, asset) {
    const id = asset.getId();

    // Get file from File Manager by `id`.
    const file = await this.getFileById(id);
    const isPrivateFile = this.isPrivate(file);
    if (!isPrivateFile && this.requestedViaPrivateEndpoint(assetRequest)) {
      asset.setOutputStrategy(new _RedirectToPublicUrlOutputStrategy.RedirectToPublicUrlOutputStrategy(assetRequest));
      return asset;
    }
    if (isPrivateFile && this.requestedViaPublicEndpoint(assetRequest)) {
      asset.setOutputStrategy(new _RedirectToPrivateUrlOutputStrategy.RedirectToPrivateUrlOutputStrategy(assetRequest));
      return asset;
    }
    console.log("file", file);
    try {
      await this.assetAuthorizer.authorize(file);
    } catch (error) {
      asset.setOutputStrategy(new _NotAuthorizedOutputStrategy.NotAuthorizedOutputStrategy());
      return asset;
    }
    const processedAsset = await this.assetProcessor.process(assetRequest, asset);
    processedAsset.setOutputStrategy(strategy => {
      return isPrivateFile ? new _PrivateCache.PrivateCache(30, strategy) : new _PublicCache.PublicCache(365, strategy);
    });
    return processedAsset;
  }

  /**
   * !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
   * This method performs a very awkward data loading and type cast which should be removed as soon as we resolve the FLP issue!
   * !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
   */
  async getFileById(id) {
    const model = await this.context.security.withoutAuthorization(() => {
      return this.context.cms.getModel("fmFile");
    });
    if (!model) {
      throw new _handlerGraphql.NotFoundError("File model not found!");
    }
    const entries = this.context.cms.storageOperations.entries;
    const storageEntry = await entries.getLatestRevisionByEntryId(model, {
      id
    });
    if (!storageEntry) {
      throw new _handlerGraphql.NotFoundError("File not found!");
    }
    const file = await (0, _apiHeadlessCms.entryFromStorageTransform)(this.context, model, storageEntry);
    return file.values;
  }
  isPrivate(file) {
    return file.accessControl && file.accessControl.type.startsWith("private-");
  }
  requestedViaPrivateEndpoint(assetRequest) {
    return assetRequest.getContext().private;
  }
  requestedViaPublicEndpoint(assetRequest) {
    return !this.requestedViaPrivateEndpoint(assetRequest);
  }
}
exports.PrivateFilesAssetProcessor = PrivateFilesAssetProcessor;

//# sourceMappingURL=PrivateFilesAssetProcessor.js.map